/*
	Modifications performed by Efabless Corporation to use automatic memory
	management. License follows:

	Copyright (C) 2023 Efabless Corporation

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

		http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/

/*
 *  yosys -- Yosys Open SYnthesis Suite
 *
 *  Copyright (C) 2012  Claire Xenia Wolf <claire@yosyshq.com>
 *
 *  Permission to use, copy, modify, and/or distribute this software for any
 *  purpose with or without fee is hereby granted, provided that the above
 *  copyright notice and this permission notice appear in all copies.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 *  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 *  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef LIBPARSE_H
#define LIBPARSE_H

#include <stdio.h>
#include <string>
#include <vector>
#include <set>
#include <memory>

namespace Yosys
{
	struct LibertyAst
	{
		std::string id, value;
		std::vector<std::string> args;
		std::vector<std::shared_ptr<LibertyAst>> children;
		~LibertyAst();
		std::shared_ptr<LibertyAst> find(std::string name);
		void dump(FILE *f, std::string indent = "", std::string path = "", bool path_ok = false);
		static std::set<std::string> blacklist;
		static std::set<std::string> whitelist;
	};

	struct LibertyParser
	{
		std::istream &f;
		int line;
		std::shared_ptr<LibertyAst> ast;
		LibertyParser(std::istream &f) : f(f), line(1), ast(parse()) {}
        
        /* lexer return values:
           'v': identifier, string, array range [...] -> str holds the token string
           'n': newline
           anything else is a single character.
        */
		int lexer(std::string &str);
		
        std::shared_ptr<LibertyAst> parse();
		void error();
        void error(const std::string &str);
	};
}

#endif

